/*The Gobbler - A tool to audit DHCP networks
 *Copyright (C) 2002 Steven Jones root@networkpenetration.com
 *www.networkpenetration.com
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "gobbler.h"
pthread_mutex_t arplock = PTHREAD_MUTEX_INITIALIZER; //lock to ensure arp scan increments by one each time
static pthread_once_t endarpscanlock = PTHREAD_ONCE_INIT;

u_int32_t acount;
int timeloop;

extern int gobblearp;
extern int specarp;
extern int bcastarp;
extern int wrongarp;
extern int passivearp;
extern int subbcastarp;
extern int dhcpreplyinfo;
extern int dhcpgobblestarted;
extern int dhcpgobbling;
extern int phosts;
extern int arpscanstarted;
extern int warning;
extern int fast;

extern libnet_t *l;
extern u_int endwait;
extern u_int32_t maskcount;
extern pthread_mutex_t packetlock;
extern struct timeval scantime;


void startarpscan(void)
{
	void thread_arpscan(int);
	extern void startdhcpdiscover(void);
	extern int checkdhcp(int);

	int i, tcount;
	
	acount = 0;
	if(passivearp){
		printf("Detecting mac addresses via passive sniffing\n\n");
		printf("sit back and wait for traffic...\n");
		//printf("note... you will not see much traffic if you are on a switch\n\n");
		if((gettimeofday(&scantime,NULL)) != 0){
			printf("Opps: gettimeofday error\n\n");
			exit(1);
		}
		arpscanstarted = 1;
		//sniffer threads handle the rest so sleep
		pause();
	}
	aptr = calloc(MAXARPSCANHANDLER, sizeof(athread));
	
	if(maskcount < MAXARPSCANHANDLER){
		tcount = maskcount + 1;
	}
	else tcount = MAXARPSCANHANDLER;
	
	if(gobblearp){
		printf("Trying to gobble an ip address...\n");
		dhcpreplyinfo = 1;
		dhcpgobblestarted = 1;
		dhcpgobbling = 1;
		startdhcpdiscover();
		phosts = 1;
		sleep(5);
		if(checkdhcp(0) != 0){
			printf("Opps: checkip ll error\n\n");
			exit(1);
		}
		dhcpreplyinfo = 0;
		dhcpgobblestarted = 0;
		dhcpgobbling = 0;
	}
	
	tcount = 1;
	for(i=0; i<tcount; i++){
		thread_arpscan(i);
	}
	pause();
}
 
void thread_arpscan(int i)
{
	void *arpscan(void *);
	arpscanstarted = 1;
	if(pthread_create(&aptr[i].thread_tid, NULL, &arpscan, (void *) i) < 0){
		printf("Opps: pthread_create error\n");
		exit(1);
	}
}

void arpscan(void *tnum)
{
	void addstats(struct libnet_stats);
        void end_arpscan(void);
        libnet_ptag_t t;
        u_long i,j; //src + dst ip addresses
        int c = 0;
	int d = 0;
	int maxloop = 0;
	PMT p_curr;
	u_char *sourceip;
        u_char enet_src[6]; 
        u_char enet_dst[6] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
	static struct timespec tv;
	//printf("maskcount %d, fast %d\n", maskcount, fast);
	if(maskcount < 257){
		//class C 
		tv.tv_sec = 1;
		tv.tv_nsec = 0;
		maxloop = 30;
	}
	else if(fast){
		tv.tv_sec = 0;
		tv.tv_nsec = 50000;
		maxloop = 40; 
	}
	
	else {
		//not class C and not fast scan
		tv.tv_sec = 0;
		tv.tv_nsec = 100000;
		maxloop = 10;
	}
	

        if(pthread_detach(pthread_self()) <0){
        	printf("Opps: pthread_detach error\n");
                pthread_exit(&status);
        }
        
	if(bcastarp){
		for(d=0;d<6;d++){
			enet_src[d] = 0x00;
		}
		i = -1;
		printf("Detecting IP addresses via broadcast ARP scan\n");
		printf("using source IP 255.255.255.255 MAC 00:00:00:00:00:00\n\n");

			if(warning){
			printf("If a NetBSD host is on the network an error occurs on the target and a\n");
			printf("message is display on the host along the lines:\n");
			printf("arplookup: unable to enter address for 255.255.255.255@00:00:00:00:00:00\n\n");
			printf("NetBSD still replies even though error\n\n");
			printf("The scan will start in 10 seconds\n");
			printf("Use -w to remove this warning, ctrl+c to cancel scan\n\n");
			sleep(10);
		}
	}

	else if(wrongarp){
		for(d=0;d<6;d++){
			enet_src[d] = 0xff;
		}
		i = -1;
		
		printf("Detecting IP addresses via wrong ARP scan\n");
		printf("Using source IP 255.255.255.255 MAC ff:ff:ff:ff:ff:ff\n\n");
		if(warning){
			printf("Win2k replies to this scan\n");
			printf("Mandrake 8 no reply\n");
			printf("OpenBSD logs a error for each packet and no reply\n");
			printf("NetBSD also logs a error for each packet and no reply\n\n");
			printf("Beware of false positives\n");
			printf("The scan will start in 10 seconds\n");
			printf("Use -w to removes this warning, ctrl+c to cancel scan\n\n");
			sleep(10);
		}
	}
	
	else if(subbcastarp){
 		u_char *bcast;
		bcast = (u_char *)&bcastaddr;
		for(d=0;d<6;d++){
			enet_src[d] = 0x00;
		}
		
		i = bcastaddr;
		printf("Detecting IP addresses via subnet broadcast ARP scan\n");
		printf("using source IP %d.%d.%d.%d MAC 00:00:00:00:00:00\n\n", bcast[0], bcast[1], bcast[2], bcast[3]);
	}

	else if(gobblearp || specarp){
		p_curr = mtpoint;
		if(p_curr == NULL){
			printf("Opps: linked list is empty top\n");
			exit(1);
		}
		for(d=0;d<6;d++){
			enet_src[d] = p_curr->srcmac[d];
		}

		i = (u_long) p_curr->ipaddress;
		sourceip = (u_char *)&(p_curr->ipaddress);
		if(gobblearp)printf("\nDetecting IP addresses via gobbler ARP scan (src IP %d.%d.%d.%d MAC %.2x:%.2x:%.2x:%.2x:%.2x:%.2x)\n\n", sourceip[0], sourceip[1], sourceip[2], sourceip[3], p_curr->srcmac[0], p_curr->srcmac[1], p_curr->srcmac[2], p_curr->srcmac[3], p_curr->srcmac[4], p_curr->srcmac[5]);
		else if(specarp)printf("Detecting IP addresses via ARP scan (src IP %d.%d.%d.%d MAC %.2x:%.2x:%.2x:%.2x:%.2x:%.2x)\n\n", sourceip[0], sourceip[1], sourceip[2], sourceip[3], p_curr->srcmac[0], p_curr->srcmac[1], p_curr->srcmac[2], p_curr->srcmac[3], p_curr->srcmac[4], p_curr->srcmac[5]);
	}
	
	else {
		//shouldn;t reach here
		printf("Opps: Unknown arp scan type\n");
		exit(1);
	}
		
	
	if((gettimeofday(&scantime,NULL)) != 0){
		printf("Opps: gettimeofday error\n\n");
		exit(1);
	}


	if(v>2)printf("Arp scan started on thread(%ld)\n", (long int) pthread_self());
 	
	while(acount <= maskcount){
		//usleep(1);
		if(pthread_mutex_lock(&packetlock) !=0){
			printf("Opps: pthread_mutex_lock (arpscan)\n");
			exit(1);
		}

		j = (localnet + htonl(acount));
		
		if(acount >= maskcount){
			pthread_once(&endarpscanlock, end_arpscan);
			pthread_exit(&status);
		}
			t = libnet_build_arp(
			ARPHRD_ETHER,   //HW type
			ETHERTYPE_IP,   //Proto
		        6,              //Hsize
		        4,              //psize
	        	ARPOP_REQUEST,  //opcode
		        enet_src,       //sender HW address
		        (u_char *)&i,   //sender proto address
	        	enet_dst,       //receiver HW address
			(u_char *)&j,   //receiver proto address
		        NULL,           //payload
	        	0,              //payload size
		        l,              //libnet handle
		       	0);             //libnet id
		
		if(t == -1){
			printf("Opps: Can;t build ARP header as %s\n\n", libnet_geterror(l));
			libnet_clear_packet(l);
			exit(1);
		}
		
		t = libnet_build_ethernet(
			enet_dst,       //enet source
	        	enet_src,       //enet dest
		        ETHERTYPE_ARP,  //proto addr
			NULL,           //payload
	        	0,              //payload size
		        l,              //libnet handle
		        0);             //libnet id
	
		if(t == -1){
			printf("Opps: Can;t build ethernet header as %s\n\n", libnet_geterror(l));
	       		libnet_clear_packet(l);
		        exit(1);
		}
	
		 if((c = libnet_write(l)) == -1){
			printf("Opps: libnet_write error (arp scan) as %s\n", libnet_geterror(l));
			exit(1);
		}
	  
		libnet_stats(l, &gs);
		libnet_clear_packet(l);
		timeloop ++;
		if(timeloop == maxloop){
			nanosleep(&tv, NULL);
			timeloop = 0;
		}
		acount++;

		if(pthread_mutex_unlock(&packetlock) !=0){
			printf("Opps: pthread_mutex_unlock (arpscan)\n");
			exit(1);
		}
	}
       	pthread_once(&endarpscanlock, end_arpscan);
	pthread_exit(&status);
}
 
 
void end_arpscan(void)
{
	void cleanup(int sigio);
	printf("\n");
	if(v) printf("Arp scan finished... waiting for %d seconds for any more replies\n", endwait);
	sleep(endwait);
	cleanup(0);
}
