/*The Gobbler - A tool to audit DHCP networks
 *Copyright (C) 2002 Steven Jones root@networkpenetration.com
 *www.networkpenetration.com
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "gobbler.h"
//start of convert functions

unsigned int convert32(u_char *ptr, int anum, int bnum, int cnum, int dnum)
{
	register unsigned int num = 0;
	register unsigned char octet;

	octet = ptr[anum];
	num = (num + octet) << 8;

	num = num & 0x0000ff00;
	octet = ptr[bnum];
	num = (num + octet) << 8;

	num = num & 0x00ffff00;
	octet = ptr[cnum];
	num = (num + octet) << 8;

	num = num & 0xffffff00;
	octet = ptr[dnum];
	num = (num + octet);

	return num;
}


unsigned int convert16(u_char *ptr, int anum, int bnum)
{
	register unsigned int suma = 0;
	register unsigned int sumb = 0;
	register unsigned int ans = 0;
	register int q = 0;
	register int p = 0;
	for(q=128;q>0;q=q/2){
		if(q &ptr[anum]){
			if((q=128 &ptr[anum])) suma+=32768;
			if((q=64 &ptr[anum])) suma+=16384;
			if((q=32 &ptr[anum])) suma+=8192;
			if((q=16 &ptr[anum])) suma+=4096;
			if((q=8 &ptr[anum])) suma+=2048;
			if((q=4 &ptr[anum])) suma+=1024;
			if((q=2 &ptr[anum])) suma+=512;
			if((q=1 &ptr[anum])) suma+=256;	
		}
	}
	
	for(p=128;p>0;p=p/2){
		if(p &ptr[bnum]){
			sumb +=p;
		}
	}
	ans = (suma + sumb);
		return(ans);
}

//Standard check sum routine 
unsigned short in_cksum(u_int16_t *ptr, int nbytes)
{
	register u_int32_t sum;
	u_int16_t oddbyte;
	register u_int16_t answer;
	/*
	 * Our algorithm is simple, using a 32-bit accumulator (sum),
	 * we add sequential 16-bit words to it, and at the end, fold back
	 * all the carry bits from the top 16 bits into the lower 16 bits.
	 */	
	sum = 0;	
	while (nbytes > 1)  {
		sum += *ptr++;
		nbytes -= 2;
	}
	/* mop up an odd byte, if necessary */
	if (nbytes == 1) {
		oddbyte = 0;            /* make sure top half is zero */
		*((u_char *) &oddbyte) = *(u_char *)ptr;   /* one byte only */
		sum += oddbyte;
	}
	/*
	 * Add back carry outs from top 16 bits to low 16 bits.
	 */
	sum  = (sum >> 16) + (sum & 0xffff);    /* add high-16 to low-16 */
	sum += (sum >> 16);                     /* add carry */
	answer = ~sum;          /* ones-complement, then truncate to 16 bits */
	return(answer);
}

unsigned int get_random_uint()
{
	int get_random_bytes(void *buf, int numbytes);	
	unsigned int i;
	get_random_bytes(&i, sizeof(unsigned int));
	return i;
}

int get_random_bytes(void *buf, int numbytes)
{
	static char bytebuf[2048];
	static char badrandomwarning = 0;
	static int bytesleft = 0;
	int res;
	int tmp;
	struct timeval tv;
	FILE *fp = NULL;
	unsigned int i;
	short *iptr;

	if (numbytes < 0 || numbytes > 0xFFFF) return -1;

	if (bytesleft == 0) {
		fp = fopen("/dev/arandom", "r");
		if (!fp) fp = fopen("/dev/urandom", "r");
		if (!fp) fp = fopen("/dev/random", "r");
		if (fp) {
			res = fread(bytebuf, 1, sizeof(bytebuf), fp);
			if (res != sizeof(bytebuf)) {
				printf("Opps: Failed to read from /dev/urandom or /dev/random");
				fclose(fp);
				fp = NULL;
			}
			bytesleft = sizeof(bytebuf);
		}
		if (!fp) {
			if (badrandomwarning == 0) {
				badrandomwarning++;
      /*      error("WARNING: your system apparently does not offer /dev/urandom or /dev/random.  Reverting to less secure version."); */

      /* Seed our random generator */
				gettimeofday(&tv, NULL);
				srand((tv.tv_sec ^ tv.tv_usec) ^ getpid());
			}

			for(i=0; i < sizeof(bytebuf) / sizeof(short); i++) {
				iptr = (short *) ((char *)bytebuf + i * sizeof(short));
				*iptr = rand();
			}
			bytesleft = (sizeof(bytebuf) / sizeof(short)) * sizeof(short);
    /*    ^^^^^^^^^^^^^^^not as meaningless as it looks  */
  		} else fclose(fp);
	}
	if (numbytes <= bytesleft) { /* we can cover it */
		memcpy(buf, bytebuf + (sizeof(bytebuf) - bytesleft), numbytes);
		bytesleft -= numbytes;
		return 0;
	}

	/* We don't have enough */
	memcpy(buf, bytebuf + (sizeof(bytebuf) - bytesleft), bytesleft);
	tmp = bytesleft;
	bytesleft = 0;
	return get_random_bytes((char *)buf + tmp, numbytes - tmp);
}

//end of convert functions

