/*The Gobbler - A tool to audit DHCP networks
 *Copyright (C) 2002 Steven Jones root@networkpenetration.com
 *www.networkpenetration.com
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


/* From RFC 2131
 *
 * 6. The client may choose to relinquish its lease on a network address
 *    by sending a DHCPRELEASE message to the server.  The client
 *    identifies the lease to be released with its 'client identifier',
 *    or 'chaddr' and network address in the DHCPRELEASE message. If the
 *    client used a 'client identifier' when it obtained the lease, it
 *    MUST use the same 'client identifier' in the DHCPRELEASE message.
*/                          

#include "gobbler.h"

pthread_mutex_t dhcpreleaselock = PTHREAD_MUTEX_INITIALIZER;

extern libnet_t *l;
extern int noicmpping;
int releasedcount = 0;

int startdhcprelease(void)
{
	void addstats(struct libnet_stats);
	int countgobbled(void);
	libnet_ptag_t t;
	libnet_ptag_t ip;
	libnet_ptag_t udp;
	libnet_ptag_t dhcp;
	u_long options_len;
        int i, a, b, c,d;
	u_char *srcip;
        u_char *options;
	u_char enet_src[6];
	u_char enet_dst[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff}; //ethernet broadcast
	PMT p_curr; 
	
	i = a = b = c = d =0;
	printf("\n");
	printf("Releasing gobbled IP addresses\n");

	noicmpping = 1; //ISC dhcpd pings addresses upon release so we don't reply ;)
	c = countgobbled();
	while(b < c){
		p_curr = mtpoint;
		if(p_curr == NULL){
			printf("Opps: No gobbled IP addresses to release\n");
			return(0);
		}
	
		while((p_curr != NULL) && (p_curr->gobbled != 1)){
			p_curr = (PMT)p_curr->nextmt;
		}
		if(p_curr == NULL){
			printf("No gobbled IP addresses to release\n");
			return(0);
		}

		else{
			srcip = (u_char *)&p_curr->ipaddress;
			for(d=0;d<6;d++){
				enet_src[d] = p_curr->srcmac[d];
			}
			//printf("releasing %d.%d.%d.%d %.2x:%.2x:%.2x:%.2x:%.2x:%.2x\n", srcip[0], srcip[1], srcip[2], srcip[3], enet_src[0], enet_src[1], enet_src[2], enet_src[3], enet_src[4], enet_src[5]);
		        i = 0;	  
			
			options = malloc(19);
			options[i++] = LIBNET_DHCP_MESSAGETYPE;
			options[i++] = 1;
			options[i++] = LIBNET_DHCP_MSGRELEASE;
			//mt.dhcptype = DHCPDISCOVER;
			
			options[i++] = 54;//server ID
			options[i++] = 4; //length
			options[i++] = p_curr->serverip[0];
			options[i++] = p_curr->serverip[1];
			options[i++] = p_curr->serverip[2];
			options[i++] = p_curr->serverip[3];
		
			options[i++] = 61; //cli ident
			options[i++] = 0x07; //length
			options[i++] = 0x01; //ehternet
			options[i++] = p_curr->srcmac[0];
			options[i++] = p_curr->srcmac[1];
			options[i++] = p_curr->srcmac[2];
			options[i++] = p_curr->srcmac[3];
			options[i++] = p_curr->srcmac[4];
			options[i++] = p_curr->srcmac[5];
			options[i++] = 0xff; //end of options
			options_len = i;	
	
			if(pthread_mutex_lock(&dhcpreleaselock) != 0){
				printf("Opps: pthread_mutex_lock error\n");
				exit(1);
			}
	
	        	dhcp = libnet_build_dhcpv4(
				LIBNET_DHCP_REQUEST,    //opcode
        		        1,                      //hardware type
		                6,                      //hardware address len
        		        0,                      //hop count
                		p_curr->transid,	//transactionid
		                0,                      //seconds since boot
				0x8000,                 //flag (broadcast)
				htonl(p_curr->ipaddress), //client IP
				0,                      //your ip
				*p_curr->serverip,                      //server ip
				0,                      //gateway ip
				enet_src,               //client hardware addr
				NULL,                   //server host name
				NULL,                   //boot file
				options,                //payload
				options_len,            //payload length
				l,                      //libnet handle
				0);                     //libnet id
        		if(dhcp == -1){
				printf("Opps: Couldn't build dhcp packet as %s\n", libnet_geterror(l));
				exit(1);
			}
		
			udp = libnet_build_udp(
				68,                     //source port
				67,                     //dst port
				LIBNET_UDP_H + LIBNET_DHCPV4_H + options_len, //packetsize
				0,                      //checksum
				NULL,                   //payload
				0,                      //payload length
				l,                      //libnet handle
		        	0);                     //libnet id
			if(udp == -1){
				printf("Opps: Couldn't build udp packet as %s\n", libnet_geterror(l));
		        	exit(1);
			}
		
	        	ip = libnet_build_ipv4(
				LIBNET_IPV4_H + LIBNET_UDP_H + LIBNET_DHCPV4_H + options_len, //length
			        0x10,                   //tos
	        		0,                      //IP ID
			        0,                      //IP Frag
			        GOBTTL,                     //TTL
	        		IPPROTO_UDP,            //protocol
		        	0,                      //checksum
			        p_curr->ipaddress,      //source ip 0.0.0.0
		        	-1,                     //inet_addr("255.255.255.255") //dest ip
			        NULL,                   //payload
		        	0,                      //length of payload
		        	l,                      //libnet handle
			        0);                     //libnet id
		
			if(ip == -1){
				printf("Opps: Couldn't build ip packet as %s\n", libnet_geterror(l));
				exit(1);
			} 
		
			t = libnet_build_ethernet(
				enet_dst,               //enet_dst
				enet_src,               //ethaddr.ether_addr_octet, //enet_src
				ETHERTYPE_IP,           //protocol type
				NULL,                   //payload
				0,                      //payload length
				l,                      //libnet handle
				0);                     //libnet id
			if(t == -1){	
				printf("Opps: Couldn't build ethernet frame as %s\n", libnet_geterror(l));
				exit(1);
			}	
		
			a = libnet_write(l);
			if(a == -1){
				printf("Opps: libnet_write error (DHCP discover) as %s\n", libnet_geterror(l));//strerror(errno));
				exit(1);
			}
	
			//printf("here\n");
			//addll(mt); //adds mac + trans to ll
			free(options);
			libnet_stats(l, &gs);
			libnet_clear_packet(l);
			p_curr->gobbled = 0;
			//printf("here  wrote packet\n");
			b++;
			//usleep(10000);
			releasedcount++;
			
			//printf("Released %d\n", releasedcount);
			if(pthread_mutex_unlock(&dhcpreleaselock) != 0){
				printf("Opps: pthread_mutex_unlock error\n");
				exit(1);
			}
		}
	}
	return(0);
}


int countgobbled(void)
{
	PMT p_curr;
	int i = 0;
	p_curr = mtpoint;
	
	if(p_curr == NULL){
		//printf("list empty\n");
		return(0);
	}
	
	while(p_curr != NULL){
		if(p_curr->gobbled == 1){
			i++;
		}
		p_curr = (PMT) p_curr->nextmt;
	}
	if(v>1)printf("countgobbled =  %d\n",i);
	return(i);

}
