/*The Gobbler - A tool to audit DHCP networks
 *Copyright (C) 2002 Steven Jones root@networkpenetration.com
 *www.networkpenetration.com
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//miscfuncs.c
#include "gobbler.h"
static pthread_once_t elock = PTHREAD_ONCE_INIT; //insures stats printed once 
extern struct timeval scantime;
struct timeval endtime;

extern int ipgobblecount;
extern int dhcpgobbling;
extern int arprecvcount;
extern int arpscanstarted;
extern int sniffer;
extern int gobblearp;
extern int nongportscan;
extern int portscan;
extern int createhost;
extern int maninthemiddlestarted;
extern int dontreleaseips;
extern int icmpechosent;
extern int icmpechorec;
extern int onlyicmpping;
extern int icmptrace;
extern int packhandlers;
extern int releasedcount;
//#ifdef BSDOS
extern int bpffd;
//#endif


void cleanup(int signo)
{
	void printstats(void);
	v = 0; //stop all sniffer out 
	pthread_once(&elock, (void *)printstats);
	exit(signo);
}

void addstats(struct libnet_stats local)
{
	gs.packets_sent += local.packets_sent;
	gs.packet_errors += local.packet_errors;
	gs.bytes_written += local.bytes_written;
}

//leeched from dug song's libdnet
//as recommened on the libnet-mailinglist
//slightly modded with file descriptor as argument
//XXX NOT USED XXX
int setsndbuf(int fd)
{
	int n;
	socklen_t len;
	len = n = 0;
	len = sizeof(n);
	if(getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &n, &len) < 0){
		printf("Opps: getsockopt failed as %s\n", strerror(errno));
		return(1);
	}
	for( n += 128; n< 1048576; n+= 128){
		if(setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &n, len) <0){
			if(errno == ENOBUFS)
				break;
			return(0);
		}
	}
	return(0);
}


void printstats(void)
{
	extern int portllcount(void);
	extern int cportllcount(void);
	extern int fportllcount(void);
	extern int ffportllcount(void);
	extern int startdhcprelease(void);
	extern int rllcount(void);
	extern int freerportll(void);
	void pr_cpu_time(void);
	void pr_getrlimits(void);
	
	int scount = 0;
	FFOPT ff_curr;
	COPT c_curr;
	int i = 0;
	int nports = 0;
	int recvpacks = 0;
	int droppacks = 0;
	int stats = 0;
	#ifndef LINUX
	struct bpf_stat bpfstat;
	#endif
	
	
	fflush(stdout);

	#ifndef LINUX
        if (ioctl(bpffd, BIOCGSTATS, (caddr_t)&bpfstat) < 0) {
                if(v)printf("Opps: Couldn't get packet stats\n");
		recvpacks = 0;
		droppacks = 0;
		stats = 0;
        }
	else {
		recvpacks = bpfstat.bs_recv;
		droppacks = bpfstat.bs_drop;
		stats = 1;
	}
	#endif 

	//if(pcap_stats(pd, &stat)<0){
	//	printf("\nOpps: pcap_stats error as %s\n", pcap_geterr(pd));
	//	exit(1);
	//}
	
	for(i=0; i<packhandlers; i++){
		count += tptr[i].thread_count;
	}

	scount = (count) - (drop) - (droppacks);
	if(scount < 0) scount = count; //opps we dropped more packet than we handled
	
	//printf("ipgobblecount = %d\n",ipgobblecount);
	if(ipgobblecount && !(dhcpgobbling)){
		if(!dontreleaseips){
			printf("\n");
			if(startdhcprelease() != 0){
				printf("Opps: startdhcprelease error... exiting\n");
				exit(1);
			}
		}
	}
	if((gettimeofday(&endtime,NULL)) != 0){
		printf("Opps: gettimeofday error\n");
		exit(1);
	}

	printf("\n---Gobbler Stats---\n");
	if(stats)printf("%d/%d packets handled from filter (using %d packet handling threads)\n",count, recvpacks, MAXPACKHANDLER);
	if(stats)printf("%d successfully decoded, %d packets dropped (kernel: %d gobbler: %d)\n", scount, droppacks + drop, droppacks, drop);
	if(!sniffer && stats)printf("%ld packets sent, %ld packet errors,  %ld bytes written\n", gs.packets_sent, gs.packet_errors, gs.bytes_written);
	#ifdef LINUX
	printf("%d packets handled (using %d packet handling threads)\n", count, MAXPACKHANDLER);
	if(!sniffer)printf("%ld packets sent, %ld packet errors, %ld bytes written\n", gs.packets_sent, gs.packet_errors, gs.bytes_written);
	#endif
		
	if(arpscanstarted){
		if(gobblearp) printf("%d IP addresses gobbled, ", ipgobblecount);
		printf("%d arp replies, ", arprecvcount);
		if(!gobblearp){
			printf("scanning time %ld seconds, ", (endtime.tv_sec - scantime.tv_sec));
			printf("running time %ld seconds\n", (endtime.tv_sec - inittime.tv_sec));
		}
		else {
			printf("%ld/%ld seconds (scan/run time) \n", (endtime.tv_sec - scantime.tv_sec), (endtime.tv_sec - inittime.tv_sec));
		}
	}
	else if(portscan || nongportscan){
		nports = rllcount();
		if(!onlyicmpping && !icmptrace)printf("Ports %d open, %d/%d filtered before/after check, %d closed, %d total\n", portllcount(), fportllcount(), ffportllcount(), cportllcount(), nports);
		if(portscan)printf("%d/%d IP addresses gobbled/released, ",ipgobblecount, releasedcount);
		if(onlyicmpping){
			 printf("echo requests sent %d, echo replies %d, ", icmpechosent, icmpechorec);
			if(icmpechosent - icmpechorec < 0){
				printf("extra replies %d\n", icmpechorec - icmpechosent);
			}
			else printf("Lost %d, ", icmpechosent - icmpechorec);
		}
		printf("%ld/%ld seconds scan/run time \n", (endtime.tv_sec - scantime.tv_sec), (endtime.tv_sec - inittime.tv_sec));
	}
	if(sniffer){
		printf("Running time %ld seconds\n", (endtime.tv_sec - inittime.tv_sec));
	}
	if(ipgobblecount && !portscan) printf("%d/%d IP addresses gobbled/released\n", ipgobblecount, releasedcount);
	if(maninthemiddlestarted || createhost){
		printf("Running time %ld seconds\n", (endtime.tv_sec - inittime.tv_sec));
	}
	
	printf("\n");
	
	if(v>1){
		for(i=0; i<packhandlers; i++){
			printf("Thread %d handled %ld packets\n", i, tptr[i].thread_count);
		}
		printf("\n");
	}
	
	close(bpffd);
	fflush(stdout);
	if(nports){
		freerportll();
	}

	if(portscan || nongportscan){
		if(ffportllcount() != 0){
			//printf("freeing filtered port linked list\n");
			while(ffportpoint != NULL){
				ff_curr = ffportpoint;
				ffportpoint = (FFOPT) ffportpoint->nextport;
				free(ff_curr);
			}
		}
		if(cportllcount() != 0){
			//printf("free closed port linked list\n");
			while(cportpoint != NULL){
				c_curr = cportpoint;
				cportpoint = (COPT) cportpoint->nextport;
				free(c_curr);
			}
		}
	}
	free(tptr);
	exit(0);
}

void egg()
{
	printf("\n\t\t\tShoutz\n\n"\
		"        ./ slider ryza sleepyhead andy mina welshguy\n"\
		"\t  everyone else at astalavista.net\n\n"\
		"\t\t  polotron + rootless\n"\
		"\tfor giving me a ickle job (cheers guys)\n\n"\
		"\t\tbassintro for hosting\n"\
		"\t\t   (www.clove.net)\n\n"\
		"\t\t     lee and stan\n"\
		"\t\tfor being lee and stan\n\n"\
		"\t\tAndre Ludwig (OSX port)\n\n"\
		"\t\t\tLanmaster\n\n"\
		"\t\ttcpdump and ethereal\n\n"\
		"\t     Mike Schiffman, Doug Song\n\n"\
		"\t\t	Fyodor\n\n"\
		);
}

void usage(void)
{
	printf( "Scanning Options\n"\
		"-A <b,g,n,s,w> Arp scan (b)cast (g)obble (n)et-broadcast (s)pec* (w)rong\n"\
		"-C <g,s> Create a host (g)obble (s)pecified*\n"\
		"-D Detect DHCP service / rogue servers on network\n"\
                "-G Gobble attack -  DoS DHCP server via IP exhaustion / MAC spoofing attack\n"\
		"-M <d,l,o> DHCP mitm attack (d)ns mitm (l)eaving subnet (o)ther ip range\n"\
		"-N <IP> None gobbled SYN scan*\n"\
		"-P <IP> SYN scan using a gobbled IP address\n"\
		"-Q <IP-r,m,n,1a:2b:3c:4d:5e:6f> Src IP-MAC (r)andom (m)ulticast (n)on-spoofed\n"\
		"-R <135-139,445,a,o,s,n> Port range (a)ll (o)sstm (s)ervices (n)nmap\n"\
		"-S Start sniffer\n"\
		"-T Traceroute to target (use with -P or -N)\n"\
		"-U ICMP ping target (use with -P or -N)\n"\
		"-X Nmap OS detection (use with -P or -N)\n"\
		"-Z Port 0 OS detection (use with -P or -N)\n"\
		"\nMisc\n"\
		"-a <x> Amount of pings (use with -U)\n"\
		"-c Closed ports displayed at end of portscan (all ports opposed to 20)\n"\
		"-d Filtered ports displayed at end of portscan (all)\n"\
		"-e <x> End of scan sleep for x seconds - wait for replies (default 2)\n"\
		"-f Fast mode - possible errors with port lists\n"\
		"-g Don't release gobbled IP's (might be handy when portscanning)\n"\
		"-h Don't ICMP ping target... useful if a firewall is blocking ICMP pings\n"\
		"-i <if> Interface (use before -Q if non spoofed mac)\n"\
		"-j Jump past rescanning filtered ports (useful when scanning all ports)\n"\
		"-l <x> Size of icmp echo request (default 32)\n"\
		"-n <x> Number of spoofed source hosts used in -P and -Cg\n"\
		"-o / -O <port num> Open port on spoofed host o(tcp) (O)udp\n"\
		"-r Don't repy to ICMP ping requests\n"\
		"-s <port> Source port for SYN scanner (Default: random)\n"\
		"-t Tag mac addresses for gobbled hosts(each will end in 4e:50)\n"\
		"-u <x> Closed UDP port used in OS detection (default port 1)\n"\
		"-v Verbose (may be used 3 times for crazy amounts of debugging info)\n"\
		"-V Display linked list after every update (used when gobbling a IP address)\n"\
		"-w Remove warnings at start of various scans\n"\
		"\nExamples\n"\
		"Gobbled scan single dynamically assigned host: Gobbler -P 192.168.1.1 -R n\n"\
		"Gobbled scan multiple src hosts: Gobbler -P 192.168.3.1 -R 21-23,445 -n 4\n"\
		"Non-gobbled scan: Gobbler -N 10.0.0.1 -Q 10.0.0.50-r -Q 10.0.0.51-r -R n -f\n"\
		"Sniffer: Gobbler -i eth0 -S -v		      Arp scan: Gobbler -i fxp0 -Ag\n"\
		"Detect rogue DHCP server: Gobbler -D -i eth0  DHCP DoS: Gobbler -G -i fxp0\n\n"\
		"Note: all options with a * require -Q\n"\
		"Note: MITM -M is in the early stages of coding\n"\
		"Note: When performing a DoS attack the gobbler crashes\n\n"\
		"WARNING read README.1ST before using the Gobbler\n"\
		"If you do not understand what you are doing, do NOT use this program!\n"\
		);
	exit(1);
}

