#include "gobbler.h"
#ifdef OSXOS

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <time.h>
#include <net/bpf.h>
#include <errno.h>
#include <err.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

int MTU1;

static int
bpf_open()
{
	int     fd;
	int     n = 0;
	char    device[sizeof "/dev/bpf0000000000"];

	/* Go through all the minors and find one that isn't in use. */
	do {
		(void) snprintf(device, sizeof device, "/dev/bpf%d", n++);
		fd = open(device, O_RDWR);
	} while (fd < 0 && errno == EBUSY);

	if (fd < 0) {
		printf("Opps: %s: %s", device, strerror(errno));
		exit(1);
	}
	return fd;
}

int GetIfrMTU(char *name) {
    int fd;
    struct ifreq ifr;
    int retval;

    retval = -1;
    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if ( fd < 0) {
        printf("Couldn't create socket for MTU\n");
        return -1;
    }

    strncpy(ifr.ifr_name, name, sizeof(ifr.ifr_name));
    if (ioctl(fd, SIOCGIFMTU, &ifr) == 0)
        retval = ifr.ifr_metric;
    else
        printf("ioctl(SIOCGIFMTU)");    
    close(fd);
	return retval;
}


int open_interface(void){
	int     		fd;
	struct 			ifreq ifr;
	u_int   		dlt;
	int     		immediate;
	int				promisc;
	struct timeval	timeout;

	printf("Opening interface %s\n", device);

	fd = bpf_open();

	(void) strncpy(ifr.ifr_name, device, sizeof ifr.ifr_name);
	if (ioctl(fd, BIOCSETIF, (caddr_t) & ifr) < 0) {
		printf("Opps: BIOCSETIF: %s", strerror(errno));
		exit(1);
	}
	/* Check that the data link layer is an Ethernet; this code won't work
	 * with anything else. */
	if (ioctl(fd, BIOCGDLT, (caddr_t) & dlt) < 0) {
		printf("Opps: BIOCGDLT: %s", strerror(errno));
		exit(1);
	}
	if (dlt != DLT_EN10MB) {
		printf("Opps: %s is not an ethernet", device);
		exit(1);
	}
	/* Switch the interface into promisc mode*/
	promisc=1;
	if (ioctl(fd, BIOCPROMISC, &promisc) < 0) {
		printf("Opps: Couldn't enter promisc mode\n");
		exit(1);
	}
	if (dlt != DLT_EN10MB) {
		printf("Opps: %s is not an ethernet", device);
		exit(1);
	}	
	
	/* Set immediate mode so packets are processed as they arrive. */
	immediate = 1;
	if (ioctl(fd, BIOCIMMEDIATE, &immediate) < 0) {
		printf("Opps: BIOCIMMEDIATE: %s", strerror(errno));
		exit(1);
	}

	/* Set immediate mode so packets are processed as they arrive. */
	timeout.tv_sec=0;
	timeout.tv_usec=1;
	if (ioctl(fd, BIOCSRTIMEOUT, &timeout) < 0) {
		printf("Opps: BIOCSRTIMEOUT: %s", strerror(errno));
		exit(1);
	}


	/* Set immediate mode so packets are processed as they arrive. */
	immediate = 0;
	if (ioctl(fd, BIOCIMMEDIATE, &immediate) < 0) {
		printf("Opps: BIOCIMMEDIATE: %s", strerror(errno));
		exit(1);
	}

	MTU1=GetIfrMTU(device);
	if(MTU1==-1) MTU1=1500;

	return(fd);
}
#endif /*if OSX*/
