/*The Gobbler - A tool to audit DHCP networks
 *Copyright (C) 2002 Steven Jones root@networkpenetration.com
 *www.networkpenetration.com
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "gobbler.h"
#include "ports.h"

pthread_mutex_t portcountlock = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t createandsend = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t startscanning = PTHREAD_COND_INITIALIZER;
pthread_mutex_t startscanlock = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t dhcpresenddiscoverlock = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t dhcpresendrequestlock = PTHREAD_MUTEX_INITIALIZER;

extern pthread_mutex_t listlock;
extern pthread_mutex_t plistlock;
extern pthread_mutex_t mlock;
extern int portzerotest;

int portscanningstarted = 0;
int filterscan = 0;
int getdestmacaddr = 0;
int ipnum = 0;
int sipcount = 0;
int timeloop = 0;
int d = 0;
int randmac = 0;
int start = 0;
int astart = 0;
int droppedflag = 0;

extern int ng;
extern int noprintinfo;
extern int srcportoption;
extern int tag;
extern int portscan;
extern int nongportscan;
extern int nongportcan;
extern int ipgobblecount;
extern int fast;
extern int arpcache;
extern int displayfiltered;
extern int displayclosed;
extern int jumpfilter;
extern int portscanstarted;
extern int dhcpreplyinfo;
extern int dhcpgobblestarted;
extern int dhcpgobbling;
extern int portl;
extern int phosts;
extern int noicmpping;
extern int onlyicmpping;
extern int osdetection;
extern int pingamount;
extern int targetlocated;
extern int icmptrace;
extern int exceedmatch;
extern int suborbroad;
extern long icmplen;


u_int ssport = 0;
u_short eport = 0;
u_short sport = 0;
libnet_plist_t plist, *plist_p;

extern libnet_t *q, *l;
extern char* hostname;
extern char* portlist;

extern u_int endwait;
extern u_int16_t srcprt;
extern struct timeval scantime;

#ifdef LINUX
#define TLOOP 5 //10
#define FLOOP 2
#endif

#ifdef BSDOS
#define TLOOP 5
#define FLOOP 2
#endif

void startportscan(void)
{
	void startdhcpdiscover(void);
	void icmpscan(int, int, long);
	int portlisttopll(void);
	extern int pllcount(void);
	int getdstmac(int);
	int checkdhcp(int);
	int portlisttorandomlist(int);
	void portscanloop(void);
	int h,i,j,k,u,tcount;
	struct in_addr sa;
	u_char *s;
	u_int nports = 0;
	int t = 0;
	h = i = j = k = tcount = u = 0;
	
	//check args
	if(!inet_aton(hostname, &sa)){
		printf("\nOpps: %s is a invalid ip address\n", hostname);
		exit(1);
	}

	s = (u_char *)&sa.s_addr;
	
	if(onlyicmpping || icmptrace){
		portl = 0;
		nports = 0;
	}
	
	else if(strncasecmp(portlist, "a", sizeof(portlist)) == 0){
		for(nports = 0; nports <= 65535; nports++){
			if(addpt(nports) !=0){
				printf("Opps: add port error\n\n");
				exit(1);
			}
		}
		nports = pllcount();
		if(v)printf("All ports to be scanned total: %d\n", nports);
	}

	else if(strncasecmp(portlist, "o", sizeof(portlist)) == 0){
		for(nports = 0; nports < sizeof(tcpports) ; nports++){
			if(tcpports[nports] == 0){
				break;
			}
			else{
				if(addpt(tcpports[nports]) != 0){
					printf("Opps: add port error\n\n");
					exit(1);
				}
			}
		}
		nports = pllcount();
		if(v)printf("OSSTM ports to be scanned total: %d\n", nports); 
	}
	
	else if(strncasecmp(portlist, "s", sizeof(portlist)) == 0){
		for(nports = 0; nports <sizeof(nmapports); nports++){
			if(nmapports[nports] == 0){
				break;
			}
			else{
				if(addpt(nmapports[nports]) != 0){
					printf("Opps: add port errror\n\n");
					exit(1);
				}
			}
		}
		nports = pllcount();
		if(v)printf("Nmap service's ports to be scanned total: %d\n", nports);
	}
	
	else if(strncasecmp(portlist, "n", sizeof(portlist)) == 0){
		for(nports = 0; nports <sizeof(rnmapports); nports++){
			if(rnmapports[nports] == 0){
				break;
			}
			else{
				if(addpt(rnmapports[nports]) != 0){
					printf("Opps: add port errror\n\n");
					exit(1);
				}
			}
		}
		nports = pllcount();
		if(v)printf("Nmap ports to be scanned total: %d\n", nports); 
	}
	
	
	else{
		if(portl){
		//list of ports 1-10,15,20 format
			nports = portlisttopll(); //convert return number	
		}

		else {
			printf("Opps: port list error\n\n");
			exit(1);
		}
		if(v)printf("User defined list of ports total: %d\n", nports);
	}
	
	//randomise ports
	if(nports > 2500){
		printf("\nRandomising portlist.... make take some time with a large port range\n");
	}
	
	if(!onlyicmpping && !icmptrace){
		nports = portlisttorandomlist(nports);
		if(nports == 0){
			printf("Opps: random list creation failed\n\n");
			exit(1);
		}
	}
	
	if(portscan && onlyicmpping){
		printf("Gobbling some IP addresses for ping\n");
	}
	else if(portscan && icmptrace){
		printf("Gobbling some IP addresses for ICMP trace\n");
	}	
	else if(portscan && !onlyicmpping && !icmptrace)printf("Gobbling an IP address for use in portscanner\n");
	
	//if gobbled portscan
	if(portscan){
		dhcpreplyinfo = 1; //printf info about dhcp server
		dhcpgobblestarted = 1; //vars for gobbling
		dhcpgobbling = 1; //vars for gobbling

		for(j=0;j<phosts;j++){
			startdhcpdiscover(); //should give us some mac n ip addresses;
			usleep(200000);
			noprintinfo = 1;
		}
		h = 0;
		while((ipgobblecount != phosts) && h != 10){
			usleep(200000);
			if((k =checkdhcp(1)) != 0){
				if(k == 1) {
					printf("Opps: checkipll error\n");
					exit(1);
				}
				if(k == 2){
					h++;
				}
				if(k == 0){
					h++; //h = 10?
				}
			}
			
			h++;
		}
		
		if(checkdhcp(0) != 0){
			printf("Opps: checkipll error\n");
			exit(1);
		}

		dhcpreplyinfo = 0; //reset gobbler args
		dhcpgobblestarted = 0;
		dhcpgobbling = 0;
	}
	
	if(phosts) sipcount = phosts;
	if(ng) sipcount = ng;
	
	//get dest mac
	getdestmacaddr = 1;
	j = 0;
	if(getdstmac(j)!=0){
		printf("Opps: getdstmac error\n");
		exit(1);
	}
	getdestmacaddr = 0;
	

	portscanstarted = 1;
	sport = 0;
	ssport = 0;
	i=0;

	if(onlyicmpping){
		if(!pingamount) pingamount = 1;
		if((gettimeofday(&scantime,NULL)) != 0){
			printf("Opps: gettimeofday error\n");
			exit(1);
		}
		icmpscan(1, GOBTTL, icmplen);
		
		start = 0;
		u = 0;
		for(u=0;u<5;u++){
			if(!start){
				sleep(1);
			}
			else{
				u = 5;
			}
		}
		if(!start){
			droppedflag = 1;
			icmpscan(1, GOBTTL, icmplen);
			for(u=0;u<10;u++){
				if(start != 1){  			
					sleep(1);		
				}
				else{
					u = 10;
				}
			}
			if(!start){
				if(onlyicmpping) printf("Opps: No response from ICMP ping.... target may be down or a firewall blocking the request\n\n");
				else printf("Opps: No response from ICMP ping.... if you still want to scan the target use the -h flag\n\n");
				cleanup(0);
			}
		}
		else{	
			if(pingamount > 1){
				if(droppedflag) icmpscan(pingamount -2, GOBTTL, icmplen);
				else icmpscan(pingamount - 1, GOBTTL, icmplen);
			}
			if(fast) sleep(2); //wiat for replies
			cleanup(1);
		}
	}
	
	else if(icmptrace){
		printf("\nTracerouting to %d.%d.%d.%d via icmp echo request\n\n", s[0],s[1],s[2],s[3]);
		t = 1;
		if((gettimeofday(&scantime,NULL)) != 0){
			printf("Opps: gettimeofday error\n");
			exit(1);
		}
		while(!targetlocated || !exceedmatch){
			icmpscan(1,t, icmplen);
			if(fast) usleep(75000);
			else sleep(3);
			if(t > 31){
				printf("\nOpps: couldn't find host within 32 hops or something is blocking our probe\n");
				cleanup(0);
			}
			t++; //increase ttl
		}
	}

	else if(!noicmpping){
		icmpscan(1, GOBTTL, icmplen);
		for(u=0;u<5;u++){
			if(start != 1){
				sleep(1);
			}
			else{
				u = 5;
			}
		}
		
		if(!start){
			printf("\nNo reply from 1st ICMP ping.... retrying\n");
			icmpscan(1, GOBTTL, icmplen);
			for(u=0;u<10;u++){
				if(start != 1){
					sleep(1);
				}
				else{
					u = 10;
				}
			}
			if(!start){
				printf("Opps: No response from ICMP ping....\n");
				printf("The target may be down or a firewall may be blocking our probe (use -h to skip ping)\n");
				cleanup(0);
			}
		}
	}

	else {
		start = 1;
	}
	
	if(onlyicmpping || icmptrace){
		cleanup(0);
	}
	
	
	if(!onlyicmpping && !icmptrace) printf("Starting port scan....\n\n");
	if((gettimeofday(&scantime,NULL)) != 0){
		printf("Opps: gettimeofday error\n");
		exit(1);
	}

	portscanloop();
	for(;;) sleep(1);
}

void portscanloop(void){

	int tcpipethpacket(u_short srcport, u_long srcip, u_char *srcmac, u_short dstport, u_long dstip, u_char *dstmac, u_char flags, int options);
	void end_portscan(void);
	RPT p_curr; //random portlist ll
	PMT i_curr; //src IP ll  
	PMI m_curr; //target mac addr ll
	FOPT f_curr; //filtered ports ll
	int randnum = 0;
	int a = 0;
	struct in_addr sa;
	int timeloop = 0;
	int filterscan = 0;
	struct timespec tv;
        struct timespec tf;
        struct timespec tff;

	//normal scan timing
        tv.tv_sec = 0;
        tv.tv_nsec = 99000000;

        //fast scan
        tf.tv_sec = 0;
        tf.tv_nsec = 5000;

        //fast filtered scan
        tff.tv_sec = 0;
        tff.tv_nsec = 100000;

	p_curr = rppoint;
	m_curr = macippoint;
	i_curr = mtpoint;
	

	if(!inet_aton(hostname, &sa)){
                printf("\nOpps: %s is a invalid ip address\n", hostname);
                exit(1);
        }

	if(p_curr == NULL){
		printf("Opps: random portlist is empty (start of portscan loop)\n\n");
		exit(1);
	}
	if(i_curr == NULL){
		printf("Opps: src ip ll is empty\n\n");
		exit(1);
	}
	if(m_curr == NULL){
		printf("Opps: target mac addr is empty\n\n");
		exit(1);
	}
	if(v)printf("source ip count %d\n", sipcount);
	portscanningstarted = 1;

	//loop through source IP's
	while(p_curr != NULL){
		i_curr = mtpoint;
		randnum = 1 + (int)((float)sipcount*rand()/(RAND_MAX+1.0));
		a = 0;
		//printf("rand num = %d\n", randnum);
		
		while((i_curr != NULL) && (a != (randnum -1))){
			a++;
			i_curr = (PMT) i_curr->nextmt;
		}

		if(i_curr == NULL){
			printf("Opps: couldn't find a src IP address\n\n");
			exit(1);
		}

		if(!srcportoption){
			srcprt = libnet_get_prand(LIBNET_PRu16);
		}
		
		//send spooded syn packet 
		if(tcpipethpacket(srcprt, i_curr->ipaddress, i_curr->srcmac, p_curr->num, sa.s_addr, m_curr->srcmac, TH_SYN, 0) !=0){
			printf("Opps: tcpipethpacket failed\n\n");
			exit(1);
		}
		timeloop++;
                if(!filterscan){
                        if(timeloop == TLOOP){
                                if(!fast) nanosleep(&tv, NULL);
                                else nanosleep(&tf, NULL);
                                timeloop = 0;
                        }
                }
                else{
                        if(timeloop == FLOOP){
                                if(!fast) nanosleep(&tv, NULL);
                                else nanosleep(&tff, NULL);
                                timeloop = 0;
                        }
                }
		
		//move onto next port
		p_curr = (RPT) p_curr->nextrp; 
	}
	
	sleep(2); //wait for the replies
	parsefports();
	if(jumpfilter){
		printf("jumping filtered\n");
		end_portscan();
	}

	f_curr = fportpoint;
	if(f_curr == NULL){
		//no filtered ports
		//printf("no filtered ports\n");
		end_portscan();
	}
	
	printf("Checking for filtered ports....\n\n");
	filterscan = 1;
	while(f_curr != NULL){
		if(f_curr == NULL){
			printf("reached end of filtered port list\n\n");
			end_portscan();
		}
		i_curr = mtpoint;
                randnum = 1 + (int)((float)sipcount*rand()/(RAND_MAX+1.0));
                a = 0;
                //printf("rand num = %d\n", randnum);

                while((i_curr != NULL) && (a != (randnum -1))){
                        a++;
                        i_curr = (PMT) i_curr->nextmt;
                }

                if(i_curr == NULL){
                        printf("Opps: couldn't find a src IP address\n\n");
                        exit(1);
                }

                if(!srcportoption){
                        srcprt = libnet_get_prand(LIBNET_PRu16);
                }

                //send spooded syn packet
		//printf("FILTERED\n");
                if(tcpipethpacket(srcprt, i_curr->ipaddress, i_curr->srcmac, f_curr->num, sa.s_addr, m_curr->srcmac, TH_SYN, 0) != 0){
			printf("Opps: tcpipethpacket failed\n\n");
			exit(1);
		}

                timeloop++;
                if(timeloop == FLOOP){
                	if(!fast) nanosleep(&tv, NULL);
                        else nanosleep(&tff, NULL);
                        timeloop = 0;
                }
		f_curr = (FOPT) f_curr->nextport;
	}
	sleep(1);

	end_portscan();
	exit(1);
}

//g being gobble type... 0 exit, 1 return and continue
int checkdhcp(int g)
{
	//checks ip linked list to ensure all dhcp addresses have been assigned (eg recieved ACK)
	int resenddhcprequest(int);
	int resenddhcpdiscover(int);
	extern int startDHCPrequest(u_int32_t );
	int j,k,l;
	PMT p_curr, q_curr;
	
	p_curr = mtpoint;
	j = 0;
	k = 0;
	
	if(p_curr == NULL){
		printf("Opps: Linked list empty\n");
		exit(1);
	}
	
	while(p_curr != NULL){
		if(p_curr->lastdhcptype == ACKREC){
			j++; //number of IP's assigned
		}
		else if(v) printf("%d dhcp type = %d\n",k, p_curr->lastdhcptype);
		k++;//number of IP's in list
		
		p_curr = (PMT) p_curr->nextmt;
	}
	if(j != phosts){
		if(v)printf("Opps: Only %d/%d dhcp assigned addresses have been created... trying to fix\n",j,phosts);
		l = 0;
		q_curr = mtpoint;
		if(q_curr == NULL){
			printf("Opps: Linked list empty\n");
			exit(1);
		}
		while(q_curr != NULL){
			if(q_curr->lastdhcptype == DISCOVERSENT){
				resenddhcpdiscover(l);
				l++;
				q_curr = (PMT) q_curr->nextmt;
			}
			else if(q_curr->lastdhcptype == OFFERREC){
				startDHCPrequest(q_curr->transid);
				l++;
				//sleep(1);
				q_curr = (PMT) q_curr->nextmt;
			}
			else if(q_curr->lastdhcptype == REQUESTSENT){
				startDHCPrequest(q_curr->transid);
				l++;
				//sleep(1);
				q_curr = (PMT) q_curr->nextmt;
			}
			else if(q_curr->lastdhcptype == ACKREC){
				l++;
				q_curr = (PMT) q_curr->nextmt;
			}
			else{
				printf("Opps: Unknown lastdhcptype\n");
				//exit(1);
				return(0);
			}
		}
	}
	else return(0);
	if(g == 2) return(0);
	sleep(1);

		p_curr = mtpoint;
	j = 0;
	k = 0;
	if(p_curr == NULL){
		printf("Opps: Linked list empty\n");
		exit(1);
	}
	
	while(p_curr != NULL){
		if(p_curr->lastdhcptype == ACKREC){
			j++; //number of IP's assigned
		}
		else if(v)printf("%d dhcp type = %d\n",k, p_curr->lastdhcptype);
		k++;//number of IP's in list
		
		p_curr = (PMT) p_curr->nextmt;
	}

	if(!g){
		if(j != phosts){
			printf("Opps: The DHCP server couldn't assign enough addresses %d/%d assigned \n\n", j, phosts);
			cleanup(0);
		}
		else{
			return(0);
		}
	}
	else {
		if(g == 2) return(0);
		//sleep(5);
		//usleep(1000000);
		//g = 1
		p_curr = mtpoint;
		j = 0;
		k = 0;
		if(p_curr == NULL){
			printf("Opps: Linked list empty\n");
			exit(1);
		}
	
		while(p_curr != NULL){
			if(p_curr->lastdhcptype == ACKREC){
				j++; //number of IP's assigned
			}
			else if(v>1)printf("%d dhcp type = %d\n",k, p_curr->lastdhcptype);
			k++;//number of IP's in list
		
			p_curr = (PMT) p_curr->nextmt;
		}
		if(j != phosts) return(2);
		else return(0);
	}

	//shouldn't reach here
	return(1);
}

int resenddhcpdiscover(int num)
{
	void addstats(struct libnet_stats);
	libnet_ptag_t t;
	libnet_ptag_t ip;
	libnet_ptag_t udp;
	libnet_ptag_t dhcp;
	u_long options_len;
	u_char *options;
	mtnode mt;
	u_char enet_src[6];
	u_char enet_dst[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
	PMT p_curr;
	int a,i,x,j;
	u_int32_t transid;
	
	i = 0;
	x = 0;
	a = 0;
	j = 0;
	transid = 0;

	//printf("resend dhcp discover\n");

	p_curr = mtpoint;	
	if(p_curr == NULL){
		printf("Opps: Linked list empty\n");
		exit(1);
	}
	
	//printf("j = %d, num = %d\n", j, num);
	while(p_curr != NULL && (j !=num)){
		//printf("inloop j = %d\n",j);
		j++;
		p_curr = (PMT) p_curr->nextmt;
	}

	if(p_curr == NULL){
		printf("Opps: can't find ip in ll\n");
		exit(1);
	}
	//printf("j = %d\n",j);
	
	for(x=0;x<6;x++){
		enet_src[x] = p_curr->srcmac[x];
	}
	transid = p_curr->transid;

	options = malloc(13 * sizeof(unsigned char));
        options[i++] = LIBNET_DHCP_MESSAGETYPE;
        options[i++] = 1;
        options[i++] = LIBNET_DHCP_MSGDISCOVER;
        mt.dhcptype = DHCPDISCOVER;
	 
	 
        options[i++] = LIBNET_DHCP_PARAMREQUEST;
        options[i++] = 7;
        options[i++] = 0x01;
        options[i++] = 0x1c;
        options[i++] = 0x02;
        options[i++] = 0x03;
        options[i++] = 0x0f;
        options[i++] = 0x06;
        options[i++] = 0x0c;
	options[i++] = LIBNET_DHCP_END;
	 
	options_len = i;

        if(pthread_mutex_lock(&dhcpresenddiscoverlock) != 0){
                printf("Opps: pthread_mutex_lock error\n");
                exit(1);
        }

        dhcp = libnet_build_dhcpv4(
                LIBNET_DHCP_REQUEST,    //opcode
                1,                      //hardware type
                6,                      //hardware address len
                0,                      //hop count
                transid,                //transactionid
                0,                      //seconds since boot
                0x8000,                 //flag (broadcast)
                0,                      //client IP
                0,                      //your ip
                0,                      //server ip
                0,                      //gateway ip
                enet_src,               //client hardware addr
                NULL,                   //server host name
                NULL,                   //boot file
                options,                //payload
                options_len,            //payload length
                l,                      //libnet handle
                0);                     //libnet id
        if(dhcp == -1){
                printf("Opps: Couldn't build dhcp packet as %s\n", libnet_geterror(l));
                exit(1);
        }
		 
        udp = libnet_build_udp(
                68,                     //source port
                67,                     //dst port
                LIBNET_UDP_H + LIBNET_DHCPV4_H + options_len, //packetsize
                0,                      //checksum
                NULL,                   //payload
                0,                      //payload length
                l,                      //libnet handle
                0);                     //libnet id
        if(udp == -1){
		printf("Opps: Couldn't build udp packet as %s\n", libnet_geterror(l));
		exit(1);
	}
				 
	ip = libnet_build_ipv4(
	        LIBNET_IPV4_H + LIBNET_UDP_H + LIBNET_DHCPV4_H + options_len, //length
	        0x10,                   //tos
	        0,                      //IP ID
		0,                      //IP Frag
                GOBTTL,                 //TTL
		IPPROTO_UDP,            //protocol
		0,                      //checksum
		0,                      //source ip 0.0.0.0
		-1,                     //inet_addr("255.255.255.255") //dest ip
		NULL,                   //payload
		0,                      //length of payload
		l,                      //libnet handle
		0);                     //libnet id
	 
	if(ip == -1){
	        printf("Opps: Couldn't build ip packet as %s\n", libnet_geterror(l));
	        exit(1);
	}
	t = libnet_build_ethernet(
		enet_dst,               //enet_dst
		enet_src,               //ethaddr.ether_addr_octet, //enet_src
		ETHERTYPE_IP,           //protocol type
		NULL,                   //payload
		0,                      //payload length
		l,                      //libnet handle
		0);                     //libnet id
	if(t == -1){
		printf("Opps: Couldn't build ethernet frame as %s\n", libnet_geterror(l));
		exit(1);
	}
			 
	a = libnet_write(l);
	if(a == -1){
		printf("Opps: libnet_write error (DHCP discover) as %s\n", libnet_geterror(l));//strerror(errno));
		exit(1);
	}
						 
	updatell(mt); //adds mac + trans to ll
	free(options);
	libnet_stats(l, &gs);
	libnet_clear_packet(l);
	if(pthread_mutex_unlock(&dhcpresenddiscoverlock) != 0){
		printf("Opps: pthread_mutex_unlock error\n");
		exit(1);
	}

	//printf("end of resend dhcp discover\n");	
	return(0);
}

int getdstmac(int j)
{
	unsigned int convert32(char *, int, int ,int, int);
	int arp_gett(arp_t *, struct arp_entry *);
	int send_arpreq(u_int32_t, u_int32_t);
	int sub = 0;
	int b = 0;
	int c = 0;
	int i = 0;
	int localip = 0;
	int broadip= 0;
	struct route_entry rentry;
	struct arp_entry aentry;
	route_t *r;
	arp_t *arp;
	u_char *tmac;
	PMT p_curr;
	if(v>2)printf("Getting destination mac address\n");
	r = route_open();
	if(r == NULL){
		printf("Opps: libdnet route open error\n");
		exit(1);
	}
	
	if(addr_aton(hostname, &rentry.route_dst)< 0){
		printf("Opps: inet_aton error\n\n");
		exit(0);
	}
	
/*	if(addr_pton(hostname, &rentry.route_dst) <0){
		printf("Opps: addr_pton error\n");
		exit(0);
	}
*/	
	//printf("route.route_dst = 0x%x, bcast 0x%x, localnet 0x%x\n", rentry.route_dst.addr_ip, bcastaddr, localnet);
	
	if(rentry.route_dst.addr_ip == localnet){
		printf("\nYou are attemepting to scan the local network address\n");
		localip = 1;
	}
	
	if(rentry.route_dst.addr_ip == bcastaddr){
		printf("\nYou are attemepting to scan the local networks broadcast address\n");
		broadip = 1;
	}

	if(route_get(r, &rentry) <0){
		sub = 1; //assume ip on subnet if route get fail
		if(v)printf("Opps: route get failed assuming IP is on subnet\n");
	}
	
	
	route_close(r);
	if(sub != 1){
		if(v)printf("%s uses gateway %s\n", addr_ntoa(&rentry.route_dst), addr_ntoa(&rentry.route_gw));
		aentry.arp_pa = rentry.route_gw;
	}
	else{
		if(v)printf("%s is on local subnet\n", addr_ntoa(&rentry.route_dst));
		if(addr_pton(hostname, &aentry.arp_pa) <0){
			printf("Opps: addr_pton error\n");
			exit(1);
		}
	}
		
	if((arp = arp_open()) == NULL){
		printf("Opps: libdnet arp open error\n");
		exit(1);
	}
	p_curr = mtpoint;

	if(p_curr == NULL){
		printf("Opps: Linked list empty\n");
		exit(1);
	}
	
	while((p_curr != NULL) && (c != j)){
		p_curr = (PMT) p_curr->nextmt;
		c++;
	}

	if(p_curr == NULL){
		printf("Opps: couldn't find IP address\n");
		exit(1);
	}

	#define ARPCOUNT 5
	
	while((arp_gett(arp, &aentry) <0) && b != ARPCOUNT){
		if(localip && (b == 0)){
			printf("Checking no hosts are hiding on the local network address\n");
			suborbroad = 1;
		}
		
		if(broadip && (b == 0)){
			printf("Checking no hosts are hiding on the network broadcast address\n");
			suborbroad = 1;
		}

		if((broadip || localip) && (onlyicmpping) && (b == ARPCOUNT -1)){
			if(broadip){
				printf("No host found so using dest mac ff:ff:ff:ff:ff:ff for broadcast ping\n");
			}
			if(localip){
				printf("No host found so using dest mac ff:ff:ff:ff:ff:ff for local network address ping\n");
			} 
		}
		if(v)printf("Opps: arp_get error.... sending arp request packet (b = %d)\n",b);
		b++;
		tmac = (unsigned char *)&aentry.arp_pa.addr_ip;
		if(send_arpreq(p_curr->ipaddress, htonl(convert32(tmac,0,1,2,3))) != 0){
			printf("Opps: send_arpreq error\n");
			exit(1);
		}
		if(!broadip && !localip){
			for(i=0;i<10;i++){
				if(arpcache == 1) b = ARPCOUNT; 
				usleep(300000);
			}
		}
		else{
			for(i=0;i<10;i++){
				if(arpcache == 1) b = ARPCOUNT;
				usleep(250000);
			}
		}
	}
	if((onlyicmpping) && (broadip || localip)){
		u_char enet[6] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };	
		u_char *enetp;
		enetp = enet;
		if(localip){
			if(addmacip(localnet, enetp) !=0){
				printf("Opps: addmacip error\n");
				exit(1);
			}
		}
		
		if(broadip){
			if(addmacip(bcastaddr, enetp) !=0){
				printf("Opps: addmacip error\n");
				exit(1);
			}
		}

	}
	
	else if((b == 3) && (arpcache != 1)){
		if(rentry.route_dst.addr_ip == localnet){
			printf("Opps: Couldn;t get mac address.... no host is hiding on this network address\n");
			printf("However.... you may be able to ping this network address using -H");
		}
		else if(rentry.route_dst.addr_ip == bcastaddr){
			printf("Opps: Coulnd;t get mac address.... no host is hding on this broadcast address\n");
			printf("However.... you may be able to ping this network broadcast address using -H");
		}
		else printf("Opps: couldn't get mac address exiting....\n");
		arp_close(arp);
		cleanup(0);
	}
		
	else {
		if(rentry.route_dst.addr_ip == localnet){
			printf("Whoops.... some mofo has created a host on the local network address\n");
			printf("%s is at %.2x:%.2x:%.2x:%.2x:%.2x:%.2x \n", addr_ntoa(&aentry.arp_pa),  aentry.arp_ha.addr_data32[0], aentry.arp_ha.addr_data32[1], aentry.arp_ha.addr_data32[2], aentry.arp_ha.addr_data32[3], aentry.arp_ha.addr_data32[4], aentry.arp_ha.addr_data32[5]);
		}
		if(rentry.route_dst.addr_ip == bcastaddr){
			printf("Whoops.... some mofo has created a host on the networks broadcast address\n");
			printf("%s is at %.2x:%.2x:%.2x:%.2x:%.2x:%.2x \n", addr_ntoa(&aentry.arp_pa),  aentry.arp_ha.addr_data32[0], aentry.arp_ha.addr_data32[1], aentry.arp_ha.addr_data32[2], aentry.arp_ha.addr_data32[3], aentry.arp_ha.addr_data32[4], aentry.arp_ha.addr_data32[5]);
		}
		
		arp_close(arp);
		return(0);
		//if(v && sub)printf("\n%s at %s \n", addr_ntoa(&aentry.arp_pa), addr_ntoa(&aentry.arp_ha));
	}
	return(0);
}

//wrapper for libdnets arp_get
int arp_gett(arp_t *arp, struct arp_entry *entry)
{
	u_int32_t convert32(char *,int, int, int, int);
	PMI p_curr;
	int a;
	int arp_getfail;
	
	arp_getfail = 0;
	if(arp_get(arp, entry) <0){
		arp_getfail = 1;
		if(v>2)printf("Opps: Arp get failed (kernel hasn;t got a copy of the dst mac)\n");
	}
		
	if(arp_getfail == 1){
		p_curr = macippoint;
		if(p_curr == NULL){
			//printf("Opps: Dest MAC IP Linked List empty\n");
			return(-1);
		}
		while((p_curr != NULL) && (entry->arp_pa.addr_ip != htonl(p_curr->ipaddr))){
			p_curr = (PMI) p_curr->nextmi;
		}

		if(p_curr == NULL){
			//printf("Opps: Can't find IP in Linked list\n");
			return(-1);
		}

		for(a=0;a<6;a++){
			entry->arp_ha.addr_data32[a] = p_curr->srcmac[a];
		}
	}
	
		//copy kernel arp entry to mac linked list
	else{
		if(addmacip(htonl(entry->arp_pa.addr_ip), (u_char *)&entry->arp_ha.addr_eth) !=0){
			printf("Opps: addmacip error\n");
			exit(1);
		}
	}	
	return(0);
}
	
//send a syn packet from spoofed source	
int tcpipethpacket(u_short srcport, u_long srcip, u_char *srcmac, u_short dstport, u_long dstip, u_char *dstmac, u_char flags, int options, u_int32_t seq)
{	
	u_int32_t convert32(char *,int, int, int, int);
	libnet_ptag_t t;
	//u_int32_t seq;
	//printf("srcport %d, srcip 0x%x, dstport %d dstip 0x%x\n", srcport, srcip, dstport, dstip);
	//seq = libnet_get_prand(LIBNET_PRu32);

	if(options){
		t = libnet_build_tcp_options( 
			//octal numbers
			//003 003 00a = window scale 10
			//001 NOP
			//002 004 001 011 = max seg size 265
			//010 012 001 002 003 004 000 000 000 000 = timestamp
			//000 000 = end of options
			(u_char *)"\003\003\012\001\002\004\001\011\010\012\001\002\003\004\000\000\000\000\000\000", 
			20, //length
			l,
			0);

		if(t == -1){
			printf("Opps: libnet_build_tcp_options error as %s\n\n", libnet_geterror(l));
			exit(1);
		}

		t = libnet_build_tcp(
			srcport,		//srcport
			dstport,	//dstport
			seq,		//seq number
			0,		//ack number
			flags,		//Flags
			1024, 		//window size
			0,		//checksum
			0,		//urg pointer
			LIBNET_TCP_H + 20 ,	//packet size + options
			NULL,		//payload
			0,		//payload size
			l,		//libnet handle
			0);		//libnet id
		if(t == -1){
			printf("Opps: libnet_build_tcp error as %s\n\n", libnet_geterror(l));
			exit(1);
		}
		t = libnet_build_ipv4(
			LIBNET_TCP_H + LIBNET_IPV4_H + 20, //length
			0,		//TOS
			0,		//id,
			0,		//IP frag
			GOBTTL,		//TTL
			IPPROTO_TCP,	//protocol
			0,		//checksum
			srcip,	//srcip
			dstip,	//dstip
			NULL,		//payload
			0,		//sizeof payload
			l,		//libnet handle
			0);		//libnet id
		if(t == -1){
			printf("Opps: libnet_build_ipv4 error as %s\n\n", libnet_geterror(l));
			exit(1);
		}
	} //end of if options required
	
	else {
		t = libnet_build_tcp(
			srcport,		//srcport
			dstport,	//dstport
			seq,		//seq number
			0,		//ack number
			flags,		//Flags
			1024, 		//window size
			0,		//checksum
			0,		//urg pointer
			LIBNET_TCP_H,	//packet size
			NULL,		//payload
			0,		//payload size
			l,		//libnet handle
			0);		//libnet id
		if(t == -1){
			printf("Opps: libnet_build_tcp error as %s\n\n", libnet_geterror(l));
			exit(1);
		}
		t = libnet_build_ipv4(
			LIBNET_TCP_H + LIBNET_IPV4_H, //length
			0,		//TOS
			0,		//id,
			0,		//IP frag
			GOBTTL,		//TTL
			IPPROTO_TCP,	//protocol
			0,		//checksum
			srcip,	//srcip
			dstip,	//dstip
			NULL,		//payload
			0,		//sizeof payload
			l,		//libnet handle
			0);		//libnet id
		if(t == -1){
			printf("Opps: libnet_build_ipv4 error as %s\n\n", libnet_geterror(l));
			exit(1);
		}

	} //end of if not options wanted

	t = libnet_build_ethernet(
		dstmac, 	//dst mac
		srcmac, 	//src mac
		ETHERTYPE_IP,	//proto
		NULL,		//payload
		0,		//payload size
		l,		//libnet handle
		0);		//libnet id
	if(t == -1){
		printf("Opps: libnet_build_ethernet error as %s\n\n", libnet_geterror(l));
		exit(1);
	}
	//printf("build packet\n");	
	if(libnet_write(l) == -1){
		printf("Opps: libnet_write error (Syn scan) as %s\n",libnet_geterror(l));
		exit(1);
	}
	//printf("sent packet\n");
		
	libnet_stats(l, &gs);
	libnet_clear_packet(l);
	//printf("packet cleared\n");
	return(0);
}

int send_arpreq(u_int32_t srcip, u_int32_t dstip)
{
	libnet_ptag_t t;
	u_char enet_src[6];
	u_char enet_dst[6] = {0xff,0xff,0xff,0xff,0xff,0xff};
	u_char enet_blank[6] = {0x00,0x00,0x00,0x00,0x00,0x00};
	u_char *sip, *dip;
	PMT p_curr, p_top;
	u_char *packet;
	u_long packet_s;
	int c, x;	
	
	if(v)printf("Sending arp request for destination mac\n");
	
	p_curr = p_top =mtpoint;
	if(p_curr == NULL){
		return(0);
	}
        
	while((p_curr != NULL) && (p_curr->ipaddress != srcip)){
		p_curr = (PMT) p_curr->nextmt;
	}
	if(p_curr == NULL){
       		return(0);
       	}
        else{
		//found
 		for(x=0; x<6; x++){
                	enet_src[x] = p_curr->srcmac[x];
		}
		
		sip = (unsigned char *)&srcip;
		dip = (unsigned char *)&dstip;
		
		t = libnet_build_arp(
			ARPHRD_ETHER,
			ETHERTYPE_IP,
			6,
			4,
			ARPOP_REQUEST,
			enet_src,
			sip,
			enet_blank,	//dst mac
			dip,		//dst ip
			NULL,
			0,
			q,
			0);
		if(t == -1){
			printf("Opps: Can;t build arp packet as %s\n",libnet_geterror(q));
			exit(1);
		}
		
		t = libnet_build_ethernet(
			enet_dst,
			enet_src,
			ETHERTYPE_ARP,
			NULL,
			0,
			q,
			0);
		if(t == -1){
			printf("Opps: Can;t build ethernet header as %s\n",libnet_geterror(q));
			exit(1);
		}

		c = libnet_adv_cull_packet(q, &packet, &packet_s);
		if(c == -1){
			printf("Opps: cull error\n");
			exit(1);
		}
		c = libnet_write(q);
		if(c == -1){
		        printf("Opps: libnet_write error (arp request) as %s\n", libnet_geterror(q));
                        libnet_clear_packet(q);
                }
		 
                libnet_stats(q, &gs);
                libnet_clear_packet(q);
                if(v)printf("\nsent arp request\n");
		return(0);
	}			
	return(0);
}

int portlisttorandomlist(int nports)
{
	//gen random number
	//find random node
	//add node to new ll
	//del old rand node
	
	extern int printrll(void);
	extern int pllcount(void);
	extern int addrandport(u_short);
	PPT p_curr;
	PPT p_old;
	u_int32_t randnum, a, number, b;
	u_int16_t tempport;
	
	b = 0;
	number = nports;
	//printf("number %d, nport %d\n", number, nports);
	p_curr = ptpoint;
	while(p_curr != NULL){
		p_curr = ptpoint;
		p_old = ptpoint;
		if(p_curr == NULL){
			printf("Opps: port linked list empty top\n\n");
			exit(0);
		}
		
		a = 0;
		randnum = 1 + (u_int32_t)((float)number*rand()/(RAND_MAX+1.0));
		
		//find random node
		while((p_curr != NULL) && (a != (randnum-1))){
			//printf("a = %d, randnum = %d\n", a, randnum);
			a++;
			p_old = p_curr;
			p_curr = (PPT) p_curr->nextpt;
		}
		
		if(p_curr == NULL){
			printf("Opps: linked list error (mid search a = %d)\n\n", a);
			exit(1);
		}
		
		//add port to new linked list
		if(addrandport(p_curr->num) != 0){
			printf("Opps: addrandport error\n\n");
			exit(1);
		}
		
		tempport = p_curr->num;
		//delete current node
		if(a == 0){
			//1st entry in linked list.... change head pointer and delete 1st node
			ptpoint = (PPT)p_curr->nextpt;
			free(p_curr);
		}
		else {
			//not 1st entry so sort out linked list order and delete current node 
			p_old->nextpt = p_curr->nextpt;
			free(p_curr);
		}

		number--;
		b++;
		//printf("pllcount %d should = count %d, portnum = %d, b = %d\n",pllcount(), number, tempport, b);
		if(number == 0){
			break;
		}
	}
	if(v>2)printrll();
	return(b); //number of ports in random linked list
}


//libnets port list to linked list converter (returns number of ports)
int portlisttopll(void)
{
	extern int pllcount(void);
	u_short sport,eport, cport;
	u_short portpaircount;
	int flag = 0;
	int h = 0;
	portpaircount = 0; //init to 0
	sport = 0;
	eport = 0;
	cport = 0;
	
	
	plist_p = &plist;
	if(libnet_plist_chain_new(l, &plist_p, portlist) == -1){
		printf("Opps: Invalid port as %s\n",libnet_geterror(l));
		exit(1);
	}
	if(v>1)printf("Port list: %s\n", libnet_plist_chain_dump_string(plist_p));
	while(flag!=1){
		h = libnet_plist_chain_next_pair(plist_p, &sport, &eport);
		if(h != 1){
			if(h == 0){
				if(v>1)printf("No more port pairs... port pair count: %d\n\n", portpaircount);
				if(flag != 1){
					flag = 1;
				}
			}	
			if(h == -1){
				printf("Opps: libnet_plist_chain_next_pair error\n\n");
				exit(1);
			}
		}	
		else {
			portpaircount++;
			if(v>1)printf("pair %d: startport = %d endport = %d\n",portpaircount,sport, eport);
			while(!(sport > eport)){ 
				cport = sport++;
				if(addpt(cport) != 0){
					printf("Opps: add port to linked list error\n\n");
					exit(1);
				}
				if(cport == 65535){
					break;
				}
			}
		}
	}
	if(portpaircount == 0){
		printf("Opps: portpaircount error\n\n");
		exit(1);
	}
	if(v>2)printpll();
	if(libnet_plist_chain_free(plist_p) == -1){
		printf("Opps: libnet_plist_chain_free error\n\n");
		exit(1);
	}
	return(pllcount());
}

void parsefports(void)
{
	//loop through each port we scan
	//testing in both open and closed port lists.... if not found put in filtered list
 
	int testopen(u_short);
	int testclosed(u_short);
        RPT  p_curr;
        int stoploop = 0;           

	//printf("parse fports nports %d\n", nports);
	if(filterscan){
		printf("Opps: internal error filter scan should not have started yet\n");
		exit(1);
	}
	
	p_curr = rppoint;
	if(p_curr == NULL){
		printf("Opps: random port ll is empty\n");
		exit(1);
	}
	while (!stoploop){	
		while((p_curr != NULL) && (p_curr->fdone == 1)){
			p_curr = (RPT) p_curr->nextrp;
		}

		if(p_curr == NULL){
			//list empty
			stoploop = 1;
		}

		else {
			if(testopen(p_curr->num) !=0){
				if(testclosed(p_curr->num) !=0){
					addfport(p_curr->num);
				}
			}
			p_curr->fdone = 1;
		}
	}
}


void parseffports(void)
{
	//loop through each port we scanned
	//testing in both open and closed port lists.... if not found put in filtered list
 
        int d;
	int testopen(u_short);
	int testclosed(u_short);
        COPT c_curr;
        POPT o_curr;
        FOPT f_curr;       
        d = 0;
                         
        c_curr = cportpoint;
        o_curr = portpoint;
        f_curr = fportpoint;

	f_curr = fportpoint;
	//if(f_curr == NULL){
		//printf("Opps: no filtered ports to parse\n");
	//}
	while((f_curr != NULL)){
		//printf("testing %d\n", f_curr->num);
		if(testclosed(f_curr->num)!=0){
			if(testopen(f_curr->num)!=0){
				addffport(f_curr->num);
			}
		}
		f_curr = (FOPT) f_curr->nextport;
	}
}

int testopen(u_short port)
{
	//1 on not in list
	//0 in list
	//
	POPT o_curr;
	o_curr = portpoint;
	//printf("testing %d in openport ll\n", port);
	if(o_curr == NULL){
	//	printf("open list empty\n");
		return(1);
	}
	while((o_curr != NULL) && (o_curr->num != port)){
		o_curr = (POPT) o_curr->nextport;
	}
	if(o_curr == NULL){
	//	printf("not in open list\n");
		return(1);
	}
	
	else return(0);

}


int testclosed(u_short port)
{
	COPT c_curr;

	//printf("port = %d", port);
	c_curr = cportpoint;
	//printf("testing %d in closedport ll\n", port);
	if(c_curr == NULL){
	//	printf("closed list empty\n");
		return(1);
	}
	while(c_curr != NULL && c_curr->num != port){
		c_curr = (COPT) c_curr->nextport;
	}
	if(c_curr == NULL){
	//	printf("not in open list\n");
		return(1);
	}
	else return(0);
}

void end_portscan(void)
{
	void cleanup(int sigio);
	extern int start_osdetection(void);
	extern int ffportllcount(void);
	extern int cportllcount(void);
	
	if(endwait){		
		if(endwait != 2)printf("Port scan finished... waiting %d seconds for any more results\n", endwait);
		sleep(endwait);
	}
	else{
		printf("Port scan finished...\n");
		usleep(500);
	}
	pthread_mutex_lock(&mlock); //stop sniffet
	if(!jumpfilter){
		parseffports();
	//	printf("ffportllcount = %d\n",ffportllcount());
	}
	if(osdetection || portzerotest){
		pthread_mutex_unlock(&mlock); //start sniffer
	
		start_osdetection();
		portscan = 1; //start_osdetection sets portscan to 1 needed for stats
		pthread_mutex_lock(&mlock); //stop sniffer
	}
	
	printportll(); //print open ports
	if(displayclosed || (cportllcount() <= CLOSEDDISPLAYLIMIT)) printcportll(); //print closed
	if(displayfiltered || (ffportllcount() <= FILTERDISPLAYLIMIT)) printffportll(); //print filtered
	cleanup(0);
}

